package com.samsung.galaxydapptron;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import com.samsung.android.sdk.blockchain.CoinType;
import com.samsung.android.sdk.blockchain.ListenableFutureTask;
import com.samsung.android.sdk.blockchain.SBlockchain;
import com.samsung.android.sdk.blockchain.account.Account;
import com.samsung.android.sdk.blockchain.account.AccountManager;
import com.samsung.android.sdk.blockchain.account.tron.TronAccount;
import com.samsung.android.sdk.blockchain.coinservice.CoinNetworkInfo;
import com.samsung.android.sdk.blockchain.coinservice.CoinService;
import com.samsung.android.sdk.blockchain.coinservice.CoinServiceFactory;
import com.samsung.android.sdk.blockchain.coinservice.tron.TronPaymentType;
import com.samsung.android.sdk.blockchain.exception.SsdkUnsupportedException;
import com.samsung.android.sdk.blockchain.network.TronNetworkType;
import com.samsung.android.sdk.blockchain.ui.BaseOnSendTransactionListener;
import com.samsung.android.sdk.blockchain.ui.CucumberWebView;
import com.samsung.android.sdk.blockchain.ui.OnSendTronTransactionListener;
import com.samsung.android.sdk.blockchain.wallet.HardwareWallet;
import com.samsung.android.sdk.blockchain.wallet.HardwareWalletType;

import org.jetbrains.annotations.NotNull;

import java.math.BigInteger;
import java.util.List;
import java.util.concurrent.ExecutionException;

public class MainActivity extends AppCompatActivity {

    private static final String LOG_TAG = "Galaxy_DApp_Tron";
    private static final String FAUCET_URL = "https://www.trongrid.io/shasta";
    private static final String MARKETPLACE_URL = "https://web3js-test-app.herokuapp.com/tron.html";
    private static final CoinNetworkInfo COIN_NETWORK_INFO = new CoinNetworkInfo(CoinType.TRX, TronNetworkType.SHASTA,
            "https://api.shasta.trongrid.io");

    private SBlockchain mSBlockchain;
    private CucumberWebView dAppWebView;
    private Account defaultAccount;
    private HardwareWallet connectedHardwareWallet;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        try {
            mSBlockchain = new SBlockchain();
            mSBlockchain.initialize(this);
            dAppWebView = findViewById(R.id.dapp_web_view);
            connectToHardwareWallet();

        } catch (SsdkUnsupportedException e) {
            handlePlatformSDKUnsupportedError(e);
        }
    }

    private void handlePlatformSDKUnsupportedError(SsdkUnsupportedException e) {
        switch (e.getErrorType()) {
            case SsdkUnsupportedException.DEVICE_NOT_SUPPORTED:
                Log.e(LOG_TAG, "Device is not Supported");
                break;
            case SsdkUnsupportedException.LIBRARY_NOT_INSTALLED:
                Log.e(LOG_TAG, "Platform SDK Library is not Installed");
                break;
            case SsdkUnsupportedException.LIBRARY_UPDATE_IS_REQUIRED:
                Log.e(LOG_TAG, "Platform SDK Library Update is Needed");
                break;
            case SsdkUnsupportedException.LIBRARY_UPDATE_IS_RECOMMENDED:
                Log.e(LOG_TAG, "Platform SDK Library is Recommended");
                break;
            case SsdkUnsupportedException.VENDOR_NOT_SUPPORTED:
                Log.e(LOG_TAG, "Platform SDK is not supported on Non-Samsung Devices");
                break;
            default:
                Log.e(LOG_TAG, "Unknown Platform SDK Unsupported Error");
        }
        Log.e(LOG_TAG, "Error Code: " + e.getErrorType());
        displayToastMessage(e.getMessage());
    }

    private void connectToHardwareWallet() {
        ListenableFutureTask<HardwareWallet> connectionTask = mSBlockchain.getHardwareWalletManager().connect(HardwareWalletType.SAMSUNG, true);
        connectionTask.setCallback(new ListenableFutureTask.Callback<HardwareWallet>() {
            @Override
            public void onSuccess(HardwareWallet hardwareWallet) {
                connectedHardwareWallet = hardwareWallet;
                setupAccounts();
            }

            @Override
            public void onFailure(ExecutionException e) {
                Log.e(LOG_TAG, "Connecting to Hardware Wallet failed.");
                Log.e(LOG_TAG, e.getMessage());
            }

            @Override
            public void onCancelled(InterruptedException e) {
                Log.e(LOG_TAG, "Connecting to Hardware Wallet cancelled.");
                Log.e(LOG_TAG, e.getMessage());
            }
        });
    }

    private void setupAccounts() {
        if (connectedHardwareWallet.isConnected()) {
            AccountManager accountManager = mSBlockchain.getAccountManager();
            List<Account> accountList = accountManager.getAccounts(connectedHardwareWallet.getWalletId(), COIN_NETWORK_INFO.getCoinType(), COIN_NETWORK_INFO.getNetworkType());
            if (accountList.isEmpty()) {
                Log.d(LOG_TAG, "Account list empty.");
                generateNewAccount(accountManager);
            } else {
                defaultAccount = accountList.get(0);
                showAccountAddressOnUI(defaultAccount.getAddress());
                initializeWebView();
            }
        } else {
            displayToastMessage("Wallet not connected!");
        }
    }

    private void generateNewAccount(AccountManager accountManager) {
        accountManager.generateNewAccount(connectedHardwareWallet, COIN_NETWORK_INFO).setCallback(new ListenableFutureTask.Callback<Account>() {
            @Override
            public void onSuccess(Account newAccount) {
                defaultAccount = newAccount;
                showAccountAddressOnUI(defaultAccount.getAddress());
                initializeWebView();
                Log.d(LOG_TAG, "Account fetched.");
            }

            @Override
            public void onFailure(ExecutionException e) {
                Log.e(LOG_TAG, "Account fetching failed.");
                Log.e(LOG_TAG, e.getMessage());
            }

            @Override
            public void onCancelled(InterruptedException e) {
                Log.e(LOG_TAG, "Account fetching cancelled.");
                Log.e(LOG_TAG, e.getMessage());
            }
        });
    }

    private void initializeWebView() {

        Log.i(LOG_TAG, "Setup Webview reached.");

        runOnUiThread(() -> {
            CoinService tronCoinService = CoinServiceFactory.getCoinService(MainActivity.this, COIN_NETWORK_INFO);
            BaseOnSendTransactionListener transactionListener = new OnSendTronTransactionListener() {
                @Override
                public void onSendTransaction(@NotNull String requestID, @NotNull TronPaymentType tronPaymentType, @NotNull TronAccount fromAccount, @NotNull String toAddress,
                                              @org.jetbrains.annotations.Nullable BigInteger productPrice, @org.jetbrains.annotations.Nullable String tokenID,
                                              @org.jetbrains.annotations.Nullable BigInteger tokenValue, @org.jetbrains.annotations.Nullable String data,
                                              @org.jetbrains.annotations.Nullable BigInteger feeLimit) {
                    //Write code for what happens when transaction is sent
                    Intent paymentIntent = dAppWebView.createTronPaymentSheetActivityIntent(MainActivity.this, requestID, connectedHardwareWallet, fromAccount, toAddress, productPrice);
                    MainActivity.this.startActivityForResult(paymentIntent, 0);
                }
            };
            dAppWebView.addCoinServiceConnector(COIN_NETWORK_INFO.getCoinType(),
                    tronCoinService, defaultAccount, transactionListener);
            dAppWebView.loadUrl(MARKETPLACE_URL);
        });

        Log.i(LOG_TAG, "Load URL reached.");

    }

    private void displayToastMessage(final String message) {
        runOnUiThread(() ->
                Toast.makeText(MainActivity.this, message, Toast.LENGTH_SHORT).show()
        );
    }

    private void showAccountAddressOnUI(String accountAddress) {
        runOnUiThread(() -> {
            ProgressBar accountAddressProgressbar = MainActivity.this.findViewById(R.id.account_address_progressbar);
            accountAddressProgressbar.setVisibility(View.GONE);
            TextView accountAddressTextView = MainActivity.this.findViewById(R.id.account_address_textview);
            accountAddressTextView.setText(accountAddress);
            enableWebPageSelectionButtons();
        });
    }

    private void enableWebPageSelectionButtons() {

        Button faucetButton = MainActivity.this.findViewById(R.id.faucet_button);
        Button marketplaceButton = MainActivity.this.findViewById(R.id.marketplace_button);
        faucetButton.setEnabled(true);
        marketplaceButton.setEnabled(true);
        faucetButton.setOnClickListener(view -> dAppWebView.loadUrl(FAUCET_URL));
        marketplaceButton.setOnClickListener(view -> dAppWebView.loadUrl(MARKETPLACE_URL));

    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == 0 && data != null) {
            dAppWebView.onActivityResult(requestCode, resultCode, data);
            switch (resultCode) {
                case Activity.RESULT_OK:
                    String transactionID = data.getStringExtra("txid");
                    Log.d(LOG_TAG, "TransactionId : " + transactionID);
                    showAlertDialog("Transaction Successful with Id : " + transactionID);
                    break;
                case Activity.RESULT_CANCELED:
                    Log.d(LOG_TAG, "Transaction canceled by user.");
                    break;
                default:
                    Log.d(LOG_TAG, "Unknown Activity Result Code. Result Code: " + resultCode);
            }
        }
    }

    private void showAlertDialog(String alertMessage) {
        android.app.AlertDialog.Builder alertBuilder = new AlertDialog.Builder(this);
        alertBuilder.setMessage(alertMessage);
        alertBuilder.setPositiveButton("Close", (dialogInterface, i) -> dialogInterface.dismiss());
        alertBuilder.show();
    }

    public void onClickCopyButton(View view){
        ClipboardManager clipboardManager = (ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);
        String msg = "";
        if (defaultAccount == null ){
            msg = "Loading Accounts.Please Wait...";
        } else {
            ClipData clipData = ClipData.newPlainText("Public Address", defaultAccount.getAddress());
            clipboardManager.setPrimaryClip(clipData);
            msg = "Address Copied";
        }
        Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
    }
}